/* This file was automatically generated, do not edit. Do not check in in version management. */

package org.harctoolbox.irscrutinizer;

import java.awt.Rectangle;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import org.harctoolbox.harchardware.ir.GlobalCache;
import org.harctoolbox.harchardware.ir.IrTrans;
import org.harctoolbox.harchardware.ir.LircClient;

/**
 * This class handles the properties of the program, saved to a file between program invocations.
 */
public class Props {
    private final static boolean useXml = true;
    private static final long serialVersionUID = 1L;
    private Properties props;
    private String filename;
    private String applicationHome;
    private boolean needSave;
    private boolean wasReset = false;
    private final boolean isWindows = System.getProperty("os.name").startsWith("Windows");

    public interface IPropertyChangeListener {
        public void propertyChange(String name, Object oldValue, Object newValue);
    }

    private String ifWindows(String windows, String nonWindows) {
        return isWindows ? windows : nonWindows;
    }

    private HashMap<String,ArrayList<IPropertyChangeListener>> changeListeners;

   private void update(String key, String value) {
        if (!props.containsKey(key)) {
            if (value != null) {
                props.setProperty(key, value);
                needSave = true;
            }
        }
        changeListeners.put(key, new ArrayList<IPropertyChangeListener>());
    }

    public boolean getWasReset() {
        return wasReset;
    }

    private void setupDefaults() {
        update("bounds", null);
        update("outputFormatIndex", "0");
        update("rmasterErrPath", System.getProperty("java.io.tmpdir") + File.separator + "rmaster.err");
        update("irWidgetCapturePortName", ifWindows("COM8","/dev/ttyUSB0"));
        update("globalCacheCaptureIpName", GlobalCache.defaultGlobalCacheIP);
        update("lircMode2Command", "mode2 -H serial");
        update("arduinoPortName", ifWindows("COM14", "/dev/ttyACM0"));
        update("arduinoCapturePortName", ifWindows("COM14", "/dev/ttyACM0"));
        update("irToyPortName", ifWindows("COM13", "/dev/ttyACM0"));
        update("irToyCapturePortName", ifWindows("COM13", "/dev/ttyACM0"));
        update("captureDevice", "LIRC Mode 2");
        update("sorterOnParametrizedTable", "false");
        update("sorterOnRawTable", "false");
        update("captureStartTimeout", "3000");
        update("captureRunTimeout", "1000");
        update("captureEndingTimeout", "300");
        update("sendingTimeout", "2000");
        update("fallbackFrequency", "38000");
        update("analyzerBase", "16");
        update("ignoreEndingSilence", "false");
        update("verbose", "false");
        update("invokeRepeatFinder", "true");
        update("disregardRepeatMins", "false");
        update("exportDir", System.getProperty("java.io.tmpdir") + File.separator + "exports");
        update("defaultImportDir", applicationHome + "testfiles");
        update("exportFormatFilePath", applicationHome + "exportformats.xml");
        update("exportFormatName", "Girr");
        update("globalCacheIpName", GlobalCache.defaultGlobalCacheIP);
        update("globalCacheModule", "2");
        update("globalCachePort", "1");
        update("globalCacheTimeout", "3000");
        update("genericSerialPortDeviceName", ifWindows("COM2", "/dev/ttyS0"));
        update("helpfileUrl", (new File(applicationHome + "doc" + File.separator + "IrScrutinizer.html")).toURI().toString());
        update("glossaryUrl", (new File(applicationHome + "doc" + File.separator + "Glossary.html")).toURI().toString());
        update("irTransIpName", IrTrans.defaultIrTransIP);
        update("transmitHardware", "Audio Port");
        update("irpMasterDocUrl", (new File(applicationHome + "doc" + File.separator + "IrpMaster.html")).toURI().toString());
        update("lircIpName", LircClient.defaultLircIP);
        update("lircPort", Integer.toString(LircClient.lircDefaultPort));
        update("lookAndFeel", "0");
        update("globalCacheApiKey", "");
        update("usePopupsForErrors", "true");
        update("usePopupsForHelp", "true");
        update("printDecodesToConsole", "false");
        update("invokeDecodeIr", "true");
        update("generateRaw", "false");
        update("generateCcf", "false");
        update("exportGenerateRaw", "false");
        update("exportGenerateCcf", "true");
        update("exportGenerateUei", "false");
        update("exportGenerateParameters", "false");
        update("exportGenerateSendIr", "false");
        update("exportSendIrModule", "1");
        update("exportSendIrConnector", "1");
        update("exportSendIrCompressed", "false");
        update("exportNoRepeats", "1");
        update("translateProntoFont", "true");
        update("protocolsIniUrl", applicationHome + "protocols.ini");
        update("irpProtocolsIniPath", applicationHome + "IrpProtocols.ini");
        update("creatingUser", System.getProperty("user.name", "unknown"));
        update("rawSeparatorIndex", "0");
        update("rawNameColumn", "1");
        update("codeColumn", "2");
        update("includeTail", "false");
        update("parametricNameColumn", "1");
        update("protocolColumn", "2");
        update("dColumn", "3");
        update("sColumn", "4");
        update("fColumn", "5");
        update("parametricNumberBaseIndex", "2");
        update("parametricSeparatorIndex", "0");
        update("girrSchemaLocation", "file:///" + applicationHome + "schemas" + File.separator + "girr.xsd");
        update("girrValidate", "false");
        update("irpMasterCurrentProtocol", "nec1");
        update("exportAutomaticFilenames", "true");
        update("invokeAnalyzerOnImports", "false");
        update("useCleansed", "false");
        update("girrFatRaw", "false");
        update("girrCreateSchemaLocation", "true");
        update("girrStyleSheetUrl", "simplehtml.xsl");
        update("girrStyleSheetType", "xsl");
        update("importWaveDivideCarrier", "true");

    }

    /**
     * Resets all properties to defaults.
     * This will probably leave the program in an inconsistent state,
     * so it should be restarted directly.
     */
    public void reset() {
        props = new Properties();
        changeListeners = new HashMap<String,ArrayList<IPropertyChangeListener>>();
        setupDefaults();
        needSave = true;
        wasReset = true;
    }

    /**
     * Sets up a Props instance from system default file name.
     * @param applicationHome
     */
    public Props(String applicationHome) {
        this(null, applicationHome);
    }

    /**
     * Sets up a Props instance from a given file name.
     * @param filename File to read from and, later, save to. Need not exist.
     * @param applicationHome
     */
    public Props(String filename, String applicationHome) {
        this.applicationHome = applicationHome;
        changeListeners = new HashMap<String,ArrayList<IPropertyChangeListener>>();
        this.filename = filename;
        if (filename == null || filename.isEmpty()) {
            String dir = System.getenv("LOCALAPPDATA"); // Win Vista and later
            if (dir == null) {
                dir = System.getenv("APPDATA"); // Win < Vista
            }
            if (dir != null) {
                dir = dir + File.separator + Version.appName;
                if (!(new File(dir)).isDirectory()) {
                    boolean status = (new File(dir)).mkdirs();
                    if (!status) {
                        System.err.println("Cannot create directory " + dir + ", using home directory instead.");
                    }
                }
            }
            this.filename = (dir != null)
                    ? (dir + File.separator + Version.appName + ".properties.xml")
                    : System.getProperty("user.home") + File.separator + "." + Version.appName + ".properties.xml";
        }

        needSave = false;
        props = new Properties();
        FileInputStream f = null;

        try {
            f = new FileInputStream(this.filename);
            if (useXml)
                props.loadFromXML(f);
            else
                props.load(f);
        } catch (FileNotFoundException ex) {
            System.err.println("Property File " + this.filename + " not found, using builtin defaults.");
            setupDefaults();
            needSave = true;
        } catch (IOException ex) {
            System.err.println("Property File " + this.filename + " could not be read, using builtin defaults.");
            setupDefaults();
            needSave = true;
        } finally {
            if (f != null) {
                try {
                    f.close();
                } catch (IOException ex) {
                    System.err.println(ex.getMessage());
                }
            }
        }
        setupDefaults();
    }

    /**
     * Save instance to given file name.
     *
     * @param filename Filename to be saved to.
     * @return success of operation
     * @throws IOException
     */
    public boolean save(File filename) throws IOException {
        if (!needSave && filename.getAbsolutePath().equals((new File(this.filename)).getAbsolutePath()))
            return false;

        FileOutputStream f;
        boolean success = false;
        try {
            f = new FileOutputStream(filename);
        } catch (FileNotFoundException ex) {
            throw (ex);
        }

        try {
            if (useXml)
                props.storeToXML(f, Version.appName + " properties, feel free to hand edit if desired");
            else
                props.store(f, Version.appName + " properties, feel free to hand edit if desired");

            success = true;
            needSave = false;
        } catch (IOException ex) {
            try {
                f.close();
            } catch (IOException exx) {
                System.err.println(exx.getMessage());
            }
            throw (ex);
        }

        try {
            f.close();
        } catch (IOException ex) {
            throw (ex);
        }

        return success;
    }

    /**
     * Saves the properties to the default, stored, file name.
     *
     * @return success of operation
     * @throws IOException
     */
    public String save() throws IOException {
        boolean result = save(new File(filename));
        return result ? filename : null;
    }

    // For debugging
    private void list() {
        props.list(System.err);
    }

    private void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        ArrayList<IPropertyChangeListener> presentListeners = changeListeners.get(propertyName);
        for (IPropertyChangeListener listener : presentListeners)
            listener.propertyChange(propertyName, oldValue, newValue);
    }

    private void addPropertyChangeListener(String propertyName, IPropertyChangeListener listener) {
        ArrayList<IPropertyChangeListener> presentListeners = changeListeners.get(propertyName);
        if (!presentListeners.contains(listener))
            presentListeners.add(listener);
    }

    private void removePropertyChangeListener(String propertyName, IPropertyChangeListener listener) {
        ArrayList<IPropertyChangeListener> presentListeners = changeListeners.get(propertyName);
        if (presentListeners.contains(listener))
            presentListeners.remove(listener);
    }

/** @return Bounds of IrScrutinizer window. */
    public Rectangle getBounds() {
        String str = props.getProperty("bounds");
        if (str == null || str.isEmpty())
            return null;
        String[] arr = str.trim().split(" +");
        return new Rectangle(Integer.parseInt(arr[0]), Integer.parseInt(arr[1]),
                Integer.parseInt(arr[2]), Integer.parseInt(arr[3]));
    }

    /** @param bounds Bounds of IrScrutinizer window. */
    public void setBounds(Rectangle bounds) {
        String oldValue = props.getProperty("bounds");
        if (bounds == null)
            return;
        String newValue = String.format("%d %d %d %d", bounds.x, bounds.y, bounds.width, bounds.height);
        if (!newValue.equals(oldValue)) {
            props.setProperty("bounds", newValue);
            needSave = true;
            firePropertyChange("bounds", oldValue, newValue);
        }
    }

    public void addBoundsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("bounds", listener);
    }

    public void removeBoundsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("bounds", listener);
    }
    
    public int getOutputFormatIndex() {
        return Integer.parseInt(props.getProperty("outputFormatIndex"));
    }

    
    public void setOutputFormatIndex(int n) {
        int oldValue = Integer.parseInt(props.getProperty("outputFormatIndex"));
        if (oldValue != n) {
            props.setProperty("outputFormatIndex", Integer.toString(n));
            needSave = true;
            firePropertyChange("outputFormatIndex", oldValue, n);
        }
    }

    public void addOutputFormatIndexChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("outputFormatIndex", listener);
    }

    public void removeOutputFormatIndexChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("outputFormatIndex", listener);
    }
    /** @return Filename of logfile for protocol.ini parse errors and warning */
    public String getRmasterErrPath() {
        return props.getProperty("rmasterErrPath");
    }

    /** @param str Filename of logfile for protocol.ini parse errors and warning */
    public void setRmasterErrPath(String str) {
        String oldValue = props.getProperty("rmasterErrPath");
        if (!oldValue.equals(str)) {
            props.setProperty("rmasterErrPath", str);
            firePropertyChange("rmasterErrPath", oldValue, str);
            needSave = true;
        }
    }

    public void addRmasterErrPathChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("rmasterErrPath", listener);
    }

    public void removeRmasterErrPathChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("rmasterErrPath", listener);
    }
    /** @return Name of device to use, e.g. COM8 or /dev/ttyUSB0. */
    public String getIrWidgetCapturePortName() {
        return props.getProperty("irWidgetCapturePortName");
    }

    /** @param str Name of device to use, e.g. COM8 or /dev/ttyUSB0. */
    public void setIrWidgetCapturePortName(String str) {
        String oldValue = props.getProperty("irWidgetCapturePortName");
        if (!oldValue.equals(str)) {
            props.setProperty("irWidgetCapturePortName", str);
            firePropertyChange("irWidgetCapturePortName", oldValue, str);
            needSave = true;
        }
    }

    public void addIrWidgetCapturePortNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irWidgetCapturePortName", listener);
    }

    public void removeIrWidgetCapturePortNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irWidgetCapturePortName", listener);
    }
    /** @return IP Name or Address of GlobalCache to use (capturing). */
    public String getGlobalCacheCaptureIpName() {
        return props.getProperty("globalCacheCaptureIpName");
    }

    /** @param str IP Name or Address of GlobalCache to use (capturing). */
    public void setGlobalCacheCaptureIpName(String str) {
        String oldValue = props.getProperty("globalCacheCaptureIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("globalCacheCaptureIpName", str);
            firePropertyChange("globalCacheCaptureIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addGlobalCacheCaptureIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalCacheCaptureIpName", listener);
    }

    public void removeGlobalCacheCaptureIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalCacheCaptureIpName", listener);
    }
    /** @return Command line for the LIRC mode2 command */
    public String getLircMode2Command() {
        return props.getProperty("lircMode2Command");
    }

    /** @param str Command line for the LIRC mode2 command */
    public void setLircMode2Command(String str) {
        String oldValue = props.getProperty("lircMode2Command");
        if (!oldValue.equals(str)) {
            props.setProperty("lircMode2Command", str);
            firePropertyChange("lircMode2Command", oldValue, str);
            needSave = true;
        }
    }

    public void addLircMode2CommandChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lircMode2Command", listener);
    }

    public void removeLircMode2CommandChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lircMode2Command", listener);
    }
    /** @return Name of device to use for sending IR with the Arduino, e.g. COM1 or /dev/ttyUSB0. */
    public String getArduinoPortName() {
        return props.getProperty("arduinoPortName");
    }

    /** @param str Name of device to use for sending IR with the Arduino, e.g. COM1 or /dev/ttyUSB0. */
    public void setArduinoPortName(String str) {
        String oldValue = props.getProperty("arduinoPortName");
        if (!oldValue.equals(str)) {
            props.setProperty("arduinoPortName", str);
            firePropertyChange("arduinoPortName", oldValue, str);
            needSave = true;
        }
    }

    public void addArduinoPortNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("arduinoPortName", listener);
    }

    public void removeArduinoPortNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("arduinoPortName", listener);
    }
    /** @return Name of device to use for capturing with the Arduino, e.g. COM1 or /dev/ttyUSB0. */
    public String getArduinoCapturePortName() {
        return props.getProperty("arduinoCapturePortName");
    }

    /** @param str Name of device to use for capturing with the Arduino, e.g. COM1 or /dev/ttyUSB0. */
    public void setArduinoCapturePortName(String str) {
        String oldValue = props.getProperty("arduinoCapturePortName");
        if (!oldValue.equals(str)) {
            props.setProperty("arduinoCapturePortName", str);
            firePropertyChange("arduinoCapturePortName", oldValue, str);
            needSave = true;
        }
    }

    public void addArduinoCapturePortNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("arduinoCapturePortName", listener);
    }

    public void removeArduinoCapturePortNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("arduinoCapturePortName", listener);
    }
    /** @return Port name for IrToy */
    public String getIrToyPortName() {
        return props.getProperty("irToyPortName");
    }

    /** @param str Port name for IrToy */
    public void setIrToyPortName(String str) {
        String oldValue = props.getProperty("irToyPortName");
        if (!oldValue.equals(str)) {
            props.setProperty("irToyPortName", str);
            firePropertyChange("irToyPortName", oldValue, str);
            needSave = true;
        }
    }

    public void addIrToyPortNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irToyPortName", listener);
    }

    public void removeIrToyPortNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irToyPortName", listener);
    }
    /** @return Name of device to use, e.g. COM1 or /dev/ttyUSB0. */
    public String getIrToyCapturePortName() {
        return props.getProperty("irToyCapturePortName");
    }

    /** @param str Name of device to use, e.g. COM1 or /dev/ttyUSB0. */
    public void setIrToyCapturePortName(String str) {
        String oldValue = props.getProperty("irToyCapturePortName");
        if (!oldValue.equals(str)) {
            props.setProperty("irToyCapturePortName", str);
            firePropertyChange("irToyCapturePortName", oldValue, str);
            needSave = true;
        }
    }

    public void addIrToyCapturePortNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irToyCapturePortName", listener);
    }

    public void removeIrToyCapturePortNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irToyCapturePortName", listener);
    }
    
    public String getCaptureDevice() {
        return props.getProperty("captureDevice");
    }

    
    public void setCaptureDevice(String str) {
        String oldValue = props.getProperty("captureDevice");
        if (!oldValue.equals(str)) {
            props.setProperty("captureDevice", str);
            firePropertyChange("captureDevice", oldValue, str);
            needSave = true;
        }
    }

    public void addCaptureDeviceChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("captureDevice", listener);
    }

    public void removeCaptureDeviceChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("captureDevice", listener);
    }
    
    public boolean getSorterOnParametrizedTable() {
        return Boolean.parseBoolean(props.getProperty("sorterOnParametrizedTable"));
    }

    
    public void setSorterOnParametrizedTable(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("sorterOnParametrizedTable"));
        if (oldValue != val) {
            props.setProperty("sorterOnParametrizedTable", Boolean.toString(val));
            needSave = true;
            firePropertyChange("sorterOnParametrizedTable", oldValue, val);
        }
    }

    public void addSorterOnParametrizedTableChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("sorterOnParametrizedTable", listener);
    }

    public void removeSorterOnParametrizedTableChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("sorterOnParametrizedTable", listener);
    }
    
    public boolean getSorterOnRawTable() {
        return Boolean.parseBoolean(props.getProperty("sorterOnRawTable"));
    }

    
    public void setSorterOnRawTable(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("sorterOnRawTable"));
        if (oldValue != val) {
            props.setProperty("sorterOnRawTable", Boolean.toString(val));
            needSave = true;
            firePropertyChange("sorterOnRawTable", oldValue, val);
        }
    }

    public void addSorterOnRawTableChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("sorterOnRawTable", listener);
    }

    public void removeSorterOnRawTableChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("sorterOnRawTable", listener);
    }
    /** @return Timeout when capturing, in milliseconds. Exact semantics may depend on the hardware selected. */
    public int getCaptureStartTimeout() {
        return Integer.parseInt(props.getProperty("captureStartTimeout"));
    }

    /** @param n Timeout when capturing, in milliseconds. Exact semantics may depend on the hardware selected. */
    public void setCaptureStartTimeout(int n) {
        int oldValue = Integer.parseInt(props.getProperty("captureStartTimeout"));
        if (oldValue != n) {
            props.setProperty("captureStartTimeout", Integer.toString(n));
            needSave = true;
            firePropertyChange("captureStartTimeout", oldValue, n);
        }
    }

    public void addCaptureStartTimeoutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("captureStartTimeout", listener);
    }

    public void removeCaptureStartTimeoutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("captureStartTimeout", listener);
    }
    /** @return Maximal length of recorded signal, in milliseconds */
    public int getCaptureRunTimeout() {
        return Integer.parseInt(props.getProperty("captureRunTimeout"));
    }

    /** @param n Maximal length of recorded signal, in milliseconds */
    public void setCaptureRunTimeout(int n) {
        int oldValue = Integer.parseInt(props.getProperty("captureRunTimeout"));
        if (oldValue != n) {
            props.setProperty("captureRunTimeout", Integer.toString(n));
            needSave = true;
            firePropertyChange("captureRunTimeout", oldValue, n);
        }
    }

    public void addCaptureRunTimeoutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("captureRunTimeout", listener);
    }

    public void removeCaptureRunTimeoutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("captureRunTimeout", listener);
    }
    /** @return Timeout when signal ends, in milliseconds */
    public int getCaptureEndingTimeout() {
        return Integer.parseInt(props.getProperty("captureEndingTimeout"));
    }

    /** @param n Timeout when signal ends, in milliseconds */
    public void setCaptureEndingTimeout(int n) {
        int oldValue = Integer.parseInt(props.getProperty("captureEndingTimeout"));
        if (oldValue != n) {
            props.setProperty("captureEndingTimeout", Integer.toString(n));
            needSave = true;
            firePropertyChange("captureEndingTimeout", oldValue, n);
        }
    }

    public void addCaptureEndingTimeoutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("captureEndingTimeout", listener);
    }

    public void removeCaptureEndingTimeoutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("captureEndingTimeout", listener);
    }
    /** @return Timeout when sending signals, in milliseconds */
    public int getSendingTimeout() {
        return Integer.parseInt(props.getProperty("sendingTimeout"));
    }

    /** @param n Timeout when sending signals, in milliseconds */
    public void setSendingTimeout(int n) {
        int oldValue = Integer.parseInt(props.getProperty("sendingTimeout"));
        if (oldValue != n) {
            props.setProperty("sendingTimeout", Integer.toString(n));
            needSave = true;
            firePropertyChange("sendingTimeout", oldValue, n);
        }
    }

    public void addSendingTimeoutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("sendingTimeout", listener);
    }

    public void removeSendingTimeoutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("sendingTimeout", listener);
    }
    /** @return Fallback frequency, for use with demodulating sensors */
    public int getFallbackFrequency() {
        return Integer.parseInt(props.getProperty("fallbackFrequency"));
    }

    /** @param n Fallback frequency, for use with demodulating sensors */
    public void setFallbackFrequency(int n) {
        int oldValue = Integer.parseInt(props.getProperty("fallbackFrequency"));
        if (oldValue != n) {
            props.setProperty("fallbackFrequency", Integer.toString(n));
            needSave = true;
            firePropertyChange("fallbackFrequency", oldValue, n);
        }
    }

    public void addFallbackFrequencyChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("fallbackFrequency", listener);
    }

    public void removeFallbackFrequencyChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("fallbackFrequency", listener);
    }
    /** @return Number base for printing analyzer results */
    public int getAnalyzerBase() {
        return Integer.parseInt(props.getProperty("analyzerBase"));
    }

    /** @param n Number base for printing analyzer results */
    public void setAnalyzerBase(int n) {
        int oldValue = Integer.parseInt(props.getProperty("analyzerBase"));
        if (oldValue != n) {
            props.setProperty("analyzerBase", Integer.toString(n));
            needSave = true;
            firePropertyChange("analyzerBase", oldValue, n);
        }
    }

    public void addAnalyzerBaseChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("analyzerBase", listener);
    }

    public void removeAnalyzerBaseChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("analyzerBase", listener);
    }
    /** @return If true, the last silence in the recorded signal will not be shown in plots */
    public boolean getIgnoreEndingSilence() {
        return Boolean.parseBoolean(props.getProperty("ignoreEndingSilence"));
    }

    /** @param val If true, the last silence in the recorded signal will not be shown in plots */
    public void setIgnoreEndingSilence(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("ignoreEndingSilence"));
        if (oldValue != val) {
            props.setProperty("ignoreEndingSilence", Boolean.toString(val));
            needSave = true;
            firePropertyChange("ignoreEndingSilence", oldValue, val);
        }
    }

    public void addIgnoreEndingSilenceChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("ignoreEndingSilence", listener);
    }

    public void removeIgnoreEndingSilenceChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("ignoreEndingSilence", listener);
    }
    /** @return If true, execute some commands more verbosely */
    public boolean getVerbose() {
        return Boolean.parseBoolean(props.getProperty("verbose"));
    }

    /** @param val If true, execute some commands more verbosely */
    public void setVerbose(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("verbose"));
        if (oldValue != val) {
            props.setProperty("verbose", Boolean.toString(val));
            needSave = true;
            firePropertyChange("verbose", oldValue, val);
        }
    }

    public void addVerboseChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("verbose", listener);
    }

    public void removeVerboseChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("verbose", listener);
    }
    /** @return Determines if the repeatfinder is invoked on captures and clipboard. */
    public boolean getInvokeRepeatFinder() {
        return Boolean.parseBoolean(props.getProperty("invokeRepeatFinder"));
    }

    /** @param val Determines if the repeatfinder is invoked on captures and clipboard. */
    public void setInvokeRepeatFinder(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("invokeRepeatFinder"));
        if (oldValue != val) {
            props.setProperty("invokeRepeatFinder", Boolean.toString(val));
            needSave = true;
            firePropertyChange("invokeRepeatFinder", oldValue, val);
        }
    }

    public void addInvokeRepeatFinderChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("invokeRepeatFinder", listener);
    }

    public void removeInvokeRepeatFinderChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("invokeRepeatFinder", listener);
    }
    /** @return Value of the "disregard repeat mins" parameter. See IrpMaster documentation for the semantic. */
    public boolean getDisregardRepeatMins() {
        return Boolean.parseBoolean(props.getProperty("disregardRepeatMins"));
    }

    /** @param val Value of the "disregard repeat mins" parameter. See IrpMaster documentation for the semantic. */
    public void setDisregardRepeatMins(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("disregardRepeatMins"));
        if (oldValue != val) {
            props.setProperty("disregardRepeatMins", Boolean.toString(val));
            needSave = true;
            firePropertyChange("disregardRepeatMins", oldValue, val);
        }
    }

    public void addDisregardRepeatMinsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("disregardRepeatMins", listener);
    }

    public void removeDisregardRepeatMinsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("disregardRepeatMins", listener);
    }
    /** @return Directory to which to write exports. */
    public String getExportDir() {
        return props.getProperty("exportDir");
    }

    /** @param str Directory to which to write exports. */
    public void setExportDir(String str) {
        String oldValue = props.getProperty("exportDir");
        if (!oldValue.equals(str)) {
            props.setProperty("exportDir", str);
            firePropertyChange("exportDir", oldValue, str);
            needSave = true;
        }
    }

    public void addExportDirChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportDir", listener);
    }

    public void removeExportDirChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportDir", listener);
    }
    /** @return Just the starting directory for file selectors. */
    public String getDefaultImportDir() {
        return props.getProperty("defaultImportDir");
    }

    /** @param str Just the starting directory for file selectors. */
    public void setDefaultImportDir(String str) {
        String oldValue = props.getProperty("defaultImportDir");
        if (!oldValue.equals(str)) {
            props.setProperty("defaultImportDir", str);
            firePropertyChange("defaultImportDir", oldValue, str);
            needSave = true;
        }
    }

    public void addDefaultImportDirChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("defaultImportDir", listener);
    }

    public void removeDefaultImportDirChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("defaultImportDir", listener);
    }
    /** @return Filename of a file consisting of export format descriptions. */
    public String getExportFormatFilePath() {
        return props.getProperty("exportFormatFilePath");
    }

    /** @param str Filename of a file consisting of export format descriptions. */
    public void setExportFormatFilePath(String str) {
        String oldValue = props.getProperty("exportFormatFilePath");
        if (!oldValue.equals(str)) {
            props.setProperty("exportFormatFilePath", str);
            firePropertyChange("exportFormatFilePath", oldValue, str);
            needSave = true;
        }
    }

    public void addExportFormatFilePathChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportFormatFilePath", listener);
    }

    public void removeExportFormatFilePathChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportFormatFilePath", listener);
    }
    /** @return Name of currenty selected export file format */
    public String getExportFormatName() {
        return props.getProperty("exportFormatName");
    }

    /** @param str Name of currenty selected export file format */
    public void setExportFormatName(String str) {
        String oldValue = props.getProperty("exportFormatName");
        if (!oldValue.equals(str)) {
            props.setProperty("exportFormatName", str);
            firePropertyChange("exportFormatName", oldValue, str);
            needSave = true;
        }
    }

    public void addExportFormatNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportFormatName", listener);
    }

    public void removeExportFormatNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportFormatName", listener);
    }
    /** @return IP Name or Address of GlobalCache to use (transmitting). */
    public String getGlobalCacheIpName() {
        return props.getProperty("globalCacheIpName");
    }

    /** @param str IP Name or Address of GlobalCache to use (transmitting). */
    public void setGlobalCacheIpName(String str) {
        String oldValue = props.getProperty("globalCacheIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("globalCacheIpName", str);
            firePropertyChange("globalCacheIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addGlobalCacheIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalCacheIpName", listener);
    }

    public void removeGlobalCacheIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalCacheIpName", listener);
    }
    /** @return Module number of Global Cache to use, see its documentation. */
    public int getGlobalCacheModule() {
        return Integer.parseInt(props.getProperty("globalCacheModule"));
    }

    /** @param n Module number of Global Cache to use, see its documentation. */
    public void setGlobalCacheModule(int n) {
        int oldValue = Integer.parseInt(props.getProperty("globalCacheModule"));
        if (oldValue != n) {
            props.setProperty("globalCacheModule", Integer.toString(n));
            needSave = true;
            firePropertyChange("globalCacheModule", oldValue, n);
        }
    }

    public void addGlobalCacheModuleChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalCacheModule", listener);
    }

    public void removeGlobalCacheModuleChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalCacheModule", listener);
    }
    /** @return IR Port number of selected Global Cache module, see Global Cache documenation. */
    public int getGlobalCachePort() {
        return Integer.parseInt(props.getProperty("globalCachePort"));
    }

    /** @param n IR Port number of selected Global Cache module, see Global Cache documenation. */
    public void setGlobalCachePort(int n) {
        int oldValue = Integer.parseInt(props.getProperty("globalCachePort"));
        if (oldValue != n) {
            props.setProperty("globalCachePort", Integer.toString(n));
            needSave = true;
            firePropertyChange("globalCachePort", oldValue, n);
        }
    }

    public void addGlobalCachePortChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalCachePort", listener);
    }

    public void removeGlobalCachePortChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalCachePort", listener);
    }
    /** @return Socket timeout for GlobalCache */
    public int getGlobalCacheTimeout() {
        return Integer.parseInt(props.getProperty("globalCacheTimeout"));
    }

    /** @param n Socket timeout for GlobalCache */
    public void setGlobalCacheTimeout(int n) {
        int oldValue = Integer.parseInt(props.getProperty("globalCacheTimeout"));
        if (oldValue != n) {
            props.setProperty("globalCacheTimeout", Integer.toString(n));
            needSave = true;
            firePropertyChange("globalCacheTimeout", oldValue, n);
        }
    }

    public void addGlobalCacheTimeoutChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalCacheTimeout", listener);
    }

    public void removeGlobalCacheTimeoutChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalCacheTimeout", listener);
    }
    /** @return Port name for generic serial port */
    public String getGenericSerialPortDeviceName() {
        return props.getProperty("genericSerialPortDeviceName");
    }

    /** @param str Port name for generic serial port */
    public void setGenericSerialPortDeviceName(String str) {
        String oldValue = props.getProperty("genericSerialPortDeviceName");
        if (!oldValue.equals(str)) {
            props.setProperty("genericSerialPortDeviceName", str);
            firePropertyChange("genericSerialPortDeviceName", oldValue, str);
            needSave = true;
        }
    }

    public void addGenericSerialPortDeviceNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("genericSerialPortDeviceName", listener);
    }

    public void removeGenericSerialPortDeviceNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("genericSerialPortDeviceName", listener);
    }
    /** @return URL for help file. */
    public String getHelpfileUrl() {
        return props.getProperty("helpfileUrl");
    }

    /** @param str URL for help file. */
    public void setHelpfileUrl(String str) {
        String oldValue = props.getProperty("helpfileUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("helpfileUrl", str);
            firePropertyChange("helpfileUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addHelpfileUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("helpfileUrl", listener);
    }

    public void removeHelpfileUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("helpfileUrl", listener);
    }
    /** @return URL for help file. */
    public String getGlossaryUrl() {
        return props.getProperty("glossaryUrl");
    }

    /** @param str URL for help file. */
    public void setGlossaryUrl(String str) {
        String oldValue = props.getProperty("glossaryUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("glossaryUrl", str);
            firePropertyChange("glossaryUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addGlossaryUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("glossaryUrl", listener);
    }

    public void removeGlossaryUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("glossaryUrl", listener);
    }
    /** @return IP name or address of IRTrans unit. */
    public String getIrTransIpName() {
        return props.getProperty("irTransIpName");
    }

    /** @param str IP name or address of IRTrans unit. */
    public void setIrTransIpName(String str) {
        String oldValue = props.getProperty("irTransIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("irTransIpName", str);
            firePropertyChange("irTransIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addIrTransIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irTransIpName", listener);
    }

    public void removeIrTransIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irTransIpName", listener);
    }
    /** @return Name of selected transmit hardware. */
    public String getTransmitHardware() {
        return props.getProperty("transmitHardware");
    }

    /** @param str Name of selected transmit hardware. */
    public void setTransmitHardware(String str) {
        String oldValue = props.getProperty("transmitHardware");
        if (!oldValue.equals(str)) {
            props.setProperty("transmitHardware", str);
            firePropertyChange("transmitHardware", oldValue, str);
            needSave = true;
        }
    }

    public void addTransmitHardwareChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("transmitHardware", listener);
    }

    public void removeTransmitHardwareChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("transmitHardware", listener);
    }
    /** @return URL of IrpMaster documentation. */
    public String getIrpMasterDocUrl() {
        return props.getProperty("irpMasterDocUrl");
    }

    /** @param str URL of IrpMaster documentation. */
    public void setIrpMasterDocUrl(String str) {
        String oldValue = props.getProperty("irpMasterDocUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("irpMasterDocUrl", str);
            firePropertyChange("irpMasterDocUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addIrpMasterDocUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irpMasterDocUrl", listener);
    }

    public void removeIrpMasterDocUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irpMasterDocUrl", listener);
    }
    /** @return IP name or address of LIRC server. */
    public String getLircIpName() {
        return props.getProperty("lircIpName");
    }

    /** @param str IP name or address of LIRC server. */
    public void setLircIpName(String str) {
        String oldValue = props.getProperty("lircIpName");
        if (!oldValue.equals(str)) {
            props.setProperty("lircIpName", str);
            firePropertyChange("lircIpName", oldValue, str);
            needSave = true;
        }
    }

    public void addLircIpNameChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lircIpName", listener);
    }

    public void removeLircIpNameChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lircIpName", listener);
    }
    /** @return TCP port number of LIRC server, typically 8765. */
    public int getLircPort() {
        return Integer.parseInt(props.getProperty("lircPort"));
    }

    /** @param n TCP port number of LIRC server, typically 8765. */
    public void setLircPort(int n) {
        int oldValue = Integer.parseInt(props.getProperty("lircPort"));
        if (oldValue != n) {
            props.setProperty("lircPort", Integer.toString(n));
            needSave = true;
            firePropertyChange("lircPort", oldValue, n);
        }
    }

    public void addLircPortChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lircPort", listener);
    }

    public void removeLircPortChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lircPort", listener);
    }
    /** @return Look and feel, as integer index in table. Semantics is system dependent. */
    public int getLookAndFeel() {
        return Integer.parseInt(props.getProperty("lookAndFeel"));
    }

    /** @param n Look and feel, as integer index in table. Semantics is system dependent. */
    public void setLookAndFeel(int n) {
        int oldValue = Integer.parseInt(props.getProperty("lookAndFeel"));
        if (oldValue != n) {
            props.setProperty("lookAndFeel", Integer.toString(n));
            needSave = true;
            firePropertyChange("lookAndFeel", oldValue, n);
        }
    }

    public void addLookAndFeelChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("lookAndFeel", listener);
    }

    public void removeLookAndFeelChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("lookAndFeel", listener);
    }
    
    public String getGlobalCacheApiKey() {
        return props.getProperty("globalCacheApiKey");
    }

    
    public void setGlobalCacheApiKey(String str) {
        String oldValue = props.getProperty("globalCacheApiKey");
        if (!oldValue.equals(str)) {
            props.setProperty("globalCacheApiKey", str);
            firePropertyChange("globalCacheApiKey", oldValue, str);
            needSave = true;
        }
    }

    public void addGlobalCacheApiKeyChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("globalCacheApiKey", listener);
    }

    public void removeGlobalCacheApiKeyChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("globalCacheApiKey", listener);
    }
    /** @return If true, use popups for help. Otherwise the console will be used. */
    public boolean getUsePopupsForErrors() {
        return Boolean.parseBoolean(props.getProperty("usePopupsForErrors"));
    }

    /** @param val If true, use popups for help. Otherwise the console will be used. */
    public void setUsePopupsForErrors(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("usePopupsForErrors"));
        if (oldValue != val) {
            props.setProperty("usePopupsForErrors", Boolean.toString(val));
            needSave = true;
            firePropertyChange("usePopupsForErrors", oldValue, val);
        }
    }

    public void addUsePopupsForErrorsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("usePopupsForErrors", listener);
    }

    public void removeUsePopupsForErrorsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("usePopupsForErrors", listener);
    }
    /** @return If true, use popups for help. Otherwise the console will be used. */
    public boolean getUsePopupsForHelp() {
        return Boolean.parseBoolean(props.getProperty("usePopupsForHelp"));
    }

    /** @param val If true, use popups for help. Otherwise the console will be used. */
    public void setUsePopupsForHelp(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("usePopupsForHelp"));
        if (oldValue != val) {
            props.setProperty("usePopupsForHelp", Boolean.toString(val));
            needSave = true;
            firePropertyChange("usePopupsForHelp", oldValue, val);
        }
    }

    public void addUsePopupsForHelpChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("usePopupsForHelp", listener);
    }

    public void removeUsePopupsForHelpChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("usePopupsForHelp", listener);
    }
    /** @return If true, print all decodes to the console, in addition to other outputs. */
    public boolean getPrintDecodesToConsole() {
        return Boolean.parseBoolean(props.getProperty("printDecodesToConsole"));
    }

    /** @param val If true, print all decodes to the console, in addition to other outputs. */
    public void setPrintDecodesToConsole(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("printDecodesToConsole"));
        if (oldValue != val) {
            props.setProperty("printDecodesToConsole", Boolean.toString(val));
            needSave = true;
            firePropertyChange("printDecodesToConsole", oldValue, val);
        }
    }

    public void addPrintDecodesToConsoleChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("printDecodesToConsole", listener);
    }

    public void removePrintDecodesToConsoleChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("printDecodesToConsole", listener);
    }
    /** @return Invoke DecodeIR when importing, even when not required. */
    public boolean getInvokeDecodeIr() {
        return Boolean.parseBoolean(props.getProperty("invokeDecodeIr"));
    }

    /** @param val Invoke DecodeIR when importing, even when not required. */
    public void setInvokeDecodeIr(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("invokeDecodeIr"));
        if (oldValue != val) {
            props.setProperty("invokeDecodeIr", Boolean.toString(val));
            needSave = true;
            firePropertyChange("invokeDecodeIr", oldValue, val);
        }
    }

    public void addInvokeDecodeIrChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("invokeDecodeIr", listener);
    }

    public void removeInvokeDecodeIrChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("invokeDecodeIr", listener);
    }
    /** @return Generate raw form when importing, even when not required. */
    public boolean getGenerateRaw() {
        return Boolean.parseBoolean(props.getProperty("generateRaw"));
    }

    /** @param val Generate raw form when importing, even when not required. */
    public void setGenerateRaw(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("generateRaw"));
        if (oldValue != val) {
            props.setProperty("generateRaw", Boolean.toString(val));
            needSave = true;
            firePropertyChange("generateRaw", oldValue, val);
        }
    }

    public void addGenerateRawChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("generateRaw", listener);
    }

    public void removeGenerateRawChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("generateRaw", listener);
    }
    /** @return Generate CCF form when importing, even when not required. */
    public boolean getGenerateCcf() {
        return Boolean.parseBoolean(props.getProperty("generateCcf"));
    }

    /** @param val Generate CCF form when importing, even when not required. */
    public void setGenerateCcf(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("generateCcf"));
        if (oldValue != val) {
            props.setProperty("generateCcf", Boolean.toString(val));
            needSave = true;
            firePropertyChange("generateCcf", oldValue, val);
        }
    }

    public void addGenerateCcfChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("generateCcf", listener);
    }

    public void removeGenerateCcfChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("generateCcf", listener);
    }
    /** @return Generate raw form when importing, even when not required. */
    public boolean getExportGenerateRaw() {
        return Boolean.parseBoolean(props.getProperty("exportGenerateRaw"));
    }

    /** @param val Generate raw form when importing, even when not required. */
    public void setExportGenerateRaw(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportGenerateRaw"));
        if (oldValue != val) {
            props.setProperty("exportGenerateRaw", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportGenerateRaw", oldValue, val);
        }
    }

    public void addExportGenerateRawChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportGenerateRaw", listener);
    }

    public void removeExportGenerateRawChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportGenerateRaw", listener);
    }
    /** @return Generate CCF form when importing, even when not required. */
    public boolean getExportGenerateCcf() {
        return Boolean.parseBoolean(props.getProperty("exportGenerateCcf"));
    }

    /** @param val Generate CCF form when importing, even when not required. */
    public void setExportGenerateCcf(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportGenerateCcf"));
        if (oldValue != val) {
            props.setProperty("exportGenerateCcf", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportGenerateCcf", oldValue, val);
        }
    }

    public void addExportGenerateCcfChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportGenerateCcf", listener);
    }

    public void removeExportGenerateCcfChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportGenerateCcf", listener);
    }
    /** @return Generate UEI learned format in exports. */
    public boolean getExportGenerateUei() {
        return Boolean.parseBoolean(props.getProperty("exportGenerateUei"));
    }

    /** @param val Generate UEI learned format in exports. */
    public void setExportGenerateUei(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportGenerateUei"));
        if (oldValue != val) {
            props.setProperty("exportGenerateUei", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportGenerateUei", oldValue, val);
        }
    }

    public void addExportGenerateUeiChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportGenerateUei", listener);
    }

    public void removeExportGenerateUeiChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportGenerateUei", listener);
    }
    /** @return Generate parameters and protocol name when exporting. */
    public boolean getExportGenerateParameters() {
        return Boolean.parseBoolean(props.getProperty("exportGenerateParameters"));
    }

    /** @param val Generate parameters and protocol name when exporting. */
    public void setExportGenerateParameters(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportGenerateParameters"));
        if (oldValue != val) {
            props.setProperty("exportGenerateParameters", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportGenerateParameters", oldValue, val);
        }
    }

    public void addExportGenerateParametersChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportGenerateParameters", listener);
    }

    public void removeExportGenerateParametersChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportGenerateParameters", listener);
    }
    /** @return Generate GlobalCache sendir format in exports. */
    public boolean getExportGenerateSendIr() {
        return Boolean.parseBoolean(props.getProperty("exportGenerateSendIr"));
    }

    /** @param val Generate GlobalCache sendir format in exports. */
    public void setExportGenerateSendIr(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportGenerateSendIr"));
        if (oldValue != val) {
            props.setProperty("exportGenerateSendIr", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportGenerateSendIr", oldValue, val);
        }
    }

    public void addExportGenerateSendIrChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportGenerateSendIr", listener);
    }

    public void removeExportGenerateSendIrChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportGenerateSendIr", listener);
    }
    /** @return Default module for sendir exports */
    public int getExportSendIrModule() {
        return Integer.parseInt(props.getProperty("exportSendIrModule"));
    }

    /** @param n Default module for sendir exports */
    public void setExportSendIrModule(int n) {
        int oldValue = Integer.parseInt(props.getProperty("exportSendIrModule"));
        if (oldValue != n) {
            props.setProperty("exportSendIrModule", Integer.toString(n));
            needSave = true;
            firePropertyChange("exportSendIrModule", oldValue, n);
        }
    }

    public void addExportSendIrModuleChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportSendIrModule", listener);
    }

    public void removeExportSendIrModuleChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportSendIrModule", listener);
    }
    /** @return Default connector for sendir exports */
    public int getExportSendIrConnector() {
        return Integer.parseInt(props.getProperty("exportSendIrConnector"));
    }

    /** @param n Default connector for sendir exports */
    public void setExportSendIrConnector(int n) {
        int oldValue = Integer.parseInt(props.getProperty("exportSendIrConnector"));
        if (oldValue != n) {
            props.setProperty("exportSendIrConnector", Integer.toString(n));
            needSave = true;
            firePropertyChange("exportSendIrConnector", oldValue, n);
        }
    }

    public void addExportSendIrConnectorChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportSendIrConnector", listener);
    }

    public void removeExportSendIrConnectorChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportSendIrConnector", listener);
    }
    /** @return Use compressed form in sendir exports? */
    public boolean getExportSendIrCompressed() {
        return Boolean.parseBoolean(props.getProperty("exportSendIrCompressed"));
    }

    /** @param val Use compressed form in sendir exports? */
    public void setExportSendIrCompressed(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportSendIrCompressed"));
        if (oldValue != val) {
            props.setProperty("exportSendIrCompressed", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportSendIrCompressed", oldValue, val);
        }
    }

    public void addExportSendIrCompressedChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportSendIrCompressed", listener);
    }

    public void removeExportSendIrCompressedChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportSendIrCompressed", listener);
    }
    /** @return Repeat count for export formats supporting it. */
    public int getExportNoRepeats() {
        return Integer.parseInt(props.getProperty("exportNoRepeats"));
    }

    /** @param n Repeat count for export formats supporting it. */
    public void setExportNoRepeats(int n) {
        int oldValue = Integer.parseInt(props.getProperty("exportNoRepeats"));
        if (oldValue != n) {
            props.setProperty("exportNoRepeats", Integer.toString(n));
            needSave = true;
            firePropertyChange("exportNoRepeats", oldValue, n);
        }
    }

    public void addExportNoRepeatsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportNoRepeats", listener);
    }

    public void removeExportNoRepeatsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportNoRepeats", listener);
    }
    /** @return If true, one-character Pronto button names, likely using the Pronto font, are tranlated to readable strings. */
    public boolean getTranslateProntoFont() {
        return Boolean.parseBoolean(props.getProperty("translateProntoFont"));
    }

    /** @param val If true, one-character Pronto button names, likely using the Pronto font, are tranlated to readable strings. */
    public void setTranslateProntoFont(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("translateProntoFont"));
        if (oldValue != val) {
            props.setProperty("translateProntoFont", Boolean.toString(val));
            needSave = true;
            firePropertyChange("translateProntoFont", oldValue, val);
        }
    }

    public void addTranslateProntoFontChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("translateProntoFont", listener);
    }

    public void removeTranslateProntoFontChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("translateProntoFont", listener);
    }
    /** @return Path to protocols.ini */
    public String getProtocolsIniUrl() {
        return props.getProperty("protocolsIniUrl");
    }

    /** @param str Path to protocols.ini */
    public void setProtocolsIniUrl(String str) {
        String oldValue = props.getProperty("protocolsIniUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("protocolsIniUrl", str);
            firePropertyChange("protocolsIniUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addProtocolsIniUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("protocolsIniUrl", listener);
    }

    public void removeProtocolsIniUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("protocolsIniUrl", listener);
    }
    /** @return Path to IrpProtocols.ini */
    public String getIrpProtocolsIniPath() {
        return props.getProperty("irpProtocolsIniPath");
    }

    /** @param str Path to IrpProtocols.ini */
    public void setIrpProtocolsIniPath(String str) {
        String oldValue = props.getProperty("irpProtocolsIniPath");
        if (!oldValue.equals(str)) {
            props.setProperty("irpProtocolsIniPath", str);
            firePropertyChange("irpProtocolsIniPath", oldValue, str);
            needSave = true;
        }
    }

    public void addIrpProtocolsIniPathChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irpProtocolsIniPath", listener);
    }

    public void removeIrpProtocolsIniPathChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irpProtocolsIniPath", listener);
    }
    /** @return Name of the user, for documentation purposes */
    public String getCreatingUser() {
        return props.getProperty("creatingUser");
    }

    /** @param str Name of the user, for documentation purposes */
    public void setCreatingUser(String str) {
        String oldValue = props.getProperty("creatingUser");
        if (!oldValue.equals(str)) {
            props.setProperty("creatingUser", str);
            firePropertyChange("creatingUser", oldValue, str);
            needSave = true;
        }
    }

    public void addCreatingUserChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("creatingUser", listener);
    }

    public void removeCreatingUserChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("creatingUser", listener);
    }
    /** @return Field separator for parsing raw files. */
    public int getRawSeparatorIndex() {
        return Integer.parseInt(props.getProperty("rawSeparatorIndex"));
    }

    /** @param n Field separator for parsing raw files. */
    public void setRawSeparatorIndex(int n) {
        int oldValue = Integer.parseInt(props.getProperty("rawSeparatorIndex"));
        if (oldValue != n) {
            props.setProperty("rawSeparatorIndex", Integer.toString(n));
            needSave = true;
            firePropertyChange("rawSeparatorIndex", oldValue, n);
        }
    }

    public void addRawSeparatorIndexChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("rawSeparatorIndex", listener);
    }

    public void removeRawSeparatorIndexChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("rawSeparatorIndex", listener);
    }
    /** @return Column for name when parsing raw files. */
    public int getRawNameColumn() {
        return Integer.parseInt(props.getProperty("rawNameColumn"));
    }

    /** @param n Column for name when parsing raw files. */
    public void setRawNameColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("rawNameColumn"));
        if (oldValue != n) {
            props.setProperty("rawNameColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("rawNameColumn", oldValue, n);
        }
    }

    public void addRawNameColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("rawNameColumn", listener);
    }

    public void removeRawNameColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("rawNameColumn", listener);
    }
    /** @return Column for code when parsing raw files. */
    public int getCodeColumn() {
        return Integer.parseInt(props.getProperty("codeColumn"));
    }

    /** @param n Column for code when parsing raw files. */
    public void setCodeColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("codeColumn"));
        if (oldValue != n) {
            props.setProperty("codeColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("codeColumn", oldValue, n);
        }
    }

    public void addCodeColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("codeColumn", listener);
    }

    public void removeCodeColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("codeColumn", listener);
    }
    /** @return Include all columns after codeColumn when parsing raw files. */
    public boolean getIncludeTail() {
        return Boolean.parseBoolean(props.getProperty("includeTail"));
    }

    /** @param val Include all columns after codeColumn when parsing raw files. */
    public void setIncludeTail(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("includeTail"));
        if (oldValue != val) {
            props.setProperty("includeTail", Boolean.toString(val));
            needSave = true;
            firePropertyChange("includeTail", oldValue, val);
        }
    }

    public void addIncludeTailChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("includeTail", listener);
    }

    public void removeIncludeTailChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("includeTail", listener);
    }
    /** @return Column for name when parsing parametrized files. */
    public int getParametricNameColumn() {
        return Integer.parseInt(props.getProperty("parametricNameColumn"));
    }

    /** @param n Column for name when parsing parametrized files. */
    public void setParametricNameColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("parametricNameColumn"));
        if (oldValue != n) {
            props.setProperty("parametricNameColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("parametricNameColumn", oldValue, n);
        }
    }

    public void addParametricNameColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("parametricNameColumn", listener);
    }

    public void removeParametricNameColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("parametricNameColumn", listener);
    }
    /** @return Column for protocol name when parsing parametric files. */
    public int getProtocolColumn() {
        return Integer.parseInt(props.getProperty("protocolColumn"));
    }

    /** @param n Column for protocol name when parsing parametric files. */
    public void setProtocolColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("protocolColumn"));
        if (oldValue != n) {
            props.setProperty("protocolColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("protocolColumn", oldValue, n);
        }
    }

    public void addProtocolColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("protocolColumn", listener);
    }

    public void removeProtocolColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("protocolColumn", listener);
    }
    /** @return Column for D (device number) when parsing parametric files. */
    public int getDColumn() {
        return Integer.parseInt(props.getProperty("dColumn"));
    }

    /** @param n Column for D (device number) when parsing parametric files. */
    public void setDColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("dColumn"));
        if (oldValue != n) {
            props.setProperty("dColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("dColumn", oldValue, n);
        }
    }

    public void addDColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("dColumn", listener);
    }

    public void removeDColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("dColumn", listener);
    }
    /** @return Column for S (subdevice number) when parsing parametric files. */
    public int getSColumn() {
        return Integer.parseInt(props.getProperty("sColumn"));
    }

    /** @param n Column for S (subdevice number) when parsing parametric files. */
    public void setSColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("sColumn"));
        if (oldValue != n) {
            props.setProperty("sColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("sColumn", oldValue, n);
        }
    }

    public void addSColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("sColumn", listener);
    }

    public void removeSColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("sColumn", listener);
    }
    /** @return Column for F (function number, OBC) when parsing parametric files. */
    public int getFColumn() {
        return Integer.parseInt(props.getProperty("fColumn"));
    }

    /** @param n Column for F (function number, OBC) when parsing parametric files. */
    public void setFColumn(int n) {
        int oldValue = Integer.parseInt(props.getProperty("fColumn"));
        if (oldValue != n) {
            props.setProperty("fColumn", Integer.toString(n));
            needSave = true;
            firePropertyChange("fColumn", oldValue, n);
        }
    }

    public void addFColumnChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("fColumn", listener);
    }

    public void removeFColumnChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("fColumn", listener);
    }
    /** @return Number base (index) for interpreting parametric files. */
    public int getParametricNumberBaseIndex() {
        return Integer.parseInt(props.getProperty("parametricNumberBaseIndex"));
    }

    /** @param n Number base (index) for interpreting parametric files. */
    public void setParametricNumberBaseIndex(int n) {
        int oldValue = Integer.parseInt(props.getProperty("parametricNumberBaseIndex"));
        if (oldValue != n) {
            props.setProperty("parametricNumberBaseIndex", Integer.toString(n));
            needSave = true;
            firePropertyChange("parametricNumberBaseIndex", oldValue, n);
        }
    }

    public void addParametricNumberBaseIndexChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("parametricNumberBaseIndex", listener);
    }

    public void removeParametricNumberBaseIndexChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("parametricNumberBaseIndex", listener);
    }
    /** @return Field separator for parsing parametric files. */
    public int getParametricSeparatorIndex() {
        return Integer.parseInt(props.getProperty("parametricSeparatorIndex"));
    }

    /** @param n Field separator for parsing parametric files. */
    public void setParametricSeparatorIndex(int n) {
        int oldValue = Integer.parseInt(props.getProperty("parametricSeparatorIndex"));
        if (oldValue != n) {
            props.setProperty("parametricSeparatorIndex", Integer.toString(n));
            needSave = true;
            firePropertyChange("parametricSeparatorIndex", oldValue, n);
        }
    }

    public void addParametricSeparatorIndexChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("parametricSeparatorIndex", listener);
    }

    public void removeParametricSeparatorIndexChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("parametricSeparatorIndex", listener);
    }
    /** @return URL to schema file for girr. */
    public String getGirrSchemaLocation() {
        return props.getProperty("girrSchemaLocation");
    }

    /** @param str URL to schema file for girr. */
    public void setGirrSchemaLocation(String str) {
        String oldValue = props.getProperty("girrSchemaLocation");
        if (!oldValue.equals(str)) {
            props.setProperty("girrSchemaLocation", str);
            firePropertyChange("girrSchemaLocation", oldValue, str);
            needSave = true;
        }
    }

    public void addGirrSchemaLocationChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("girrSchemaLocation", listener);
    }

    public void removeGirrSchemaLocationChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("girrSchemaLocation", listener);
    }
    /** @return If truu, girr instances are validated against a schema. */
    public boolean getGirrValidate() {
        return Boolean.parseBoolean(props.getProperty("girrValidate"));
    }

    /** @param val If truu, girr instances are validated against a schema. */
    public void setGirrValidate(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("girrValidate"));
        if (oldValue != val) {
            props.setProperty("girrValidate", Boolean.toString(val));
            needSave = true;
            firePropertyChange("girrValidate", oldValue, val);
        }
    }

    public void addGirrValidateChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("girrValidate", listener);
    }

    public void removeGirrValidateChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("girrValidate", listener);
    }
    /** @return Selected protocol for generator. */
    public String getIrpMasterCurrentProtocol() {
        return props.getProperty("irpMasterCurrentProtocol");
    }

    /** @param str Selected protocol for generator. */
    public void setIrpMasterCurrentProtocol(String str) {
        String oldValue = props.getProperty("irpMasterCurrentProtocol");
        if (!oldValue.equals(str)) {
            props.setProperty("irpMasterCurrentProtocol", str);
            firePropertyChange("irpMasterCurrentProtocol", oldValue, str);
            needSave = true;
        }
    }

    public void addIrpMasterCurrentProtocolChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("irpMasterCurrentProtocol", listener);
    }

    public void removeIrpMasterCurrentProtocolChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("irpMasterCurrentProtocol", listener);
    }
    /** @return Generate automatic file names for exports */
    public boolean getExportAutomaticFilenames() {
        return Boolean.parseBoolean(props.getProperty("exportAutomaticFilenames"));
    }

    /** @param val Generate automatic file names for exports */
    public void setExportAutomaticFilenames(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("exportAutomaticFilenames"));
        if (oldValue != val) {
            props.setProperty("exportAutomaticFilenames", Boolean.toString(val));
            needSave = true;
            firePropertyChange("exportAutomaticFilenames", oldValue, val);
        }
    }

    public void addExportAutomaticFilenamesChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("exportAutomaticFilenames", listener);
    }

    public void removeExportAutomaticFilenamesChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("exportAutomaticFilenames", listener);
    }
    /** @return If true, the analyzer is invoked on imports */
    public boolean getInvokeAnalyzerOnImports() {
        return Boolean.parseBoolean(props.getProperty("invokeAnalyzerOnImports"));
    }

    /** @param val If true, the analyzer is invoked on imports */
    public void setInvokeAnalyzerOnImports(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("invokeAnalyzerOnImports"));
        if (oldValue != val) {
            props.setProperty("invokeAnalyzerOnImports", Boolean.toString(val));
            needSave = true;
            firePropertyChange("invokeAnalyzerOnImports", oldValue, val);
        }
    }

    public void addInvokeAnalyzerOnImportsChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("invokeAnalyzerOnImports", listener);
    }

    public void removeInvokeAnalyzerOnImportsChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("invokeAnalyzerOnImports", listener);
    }
    /** @return If true, show captured signals with repeats eliminated, as per repeatfinder. */
    public boolean getUseCleansed() {
        return Boolean.parseBoolean(props.getProperty("useCleansed"));
    }

    /** @param val If true, show captured signals with repeats eliminated, as per repeatfinder. */
    public void setUseCleansed(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("useCleansed"));
        if (oldValue != val) {
            props.setProperty("useCleansed", Boolean.toString(val));
            needSave = true;
            firePropertyChange("useCleansed", oldValue, val);
        }
    }

    public void addUseCleansedChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("useCleansed", listener);
    }

    public void removeUseCleansedChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("useCleansed", listener);
    }
    
    public boolean getGirrFatRaw() {
        return Boolean.parseBoolean(props.getProperty("girrFatRaw"));
    }

    
    public void setGirrFatRaw(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("girrFatRaw"));
        if (oldValue != val) {
            props.setProperty("girrFatRaw", Boolean.toString(val));
            needSave = true;
            firePropertyChange("girrFatRaw", oldValue, val);
        }
    }

    public void addGirrFatRawChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("girrFatRaw", listener);
    }

    public void removeGirrFatRawChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("girrFatRaw", listener);
    }
    
    public boolean getGirrCreateSchemaLocation() {
        return Boolean.parseBoolean(props.getProperty("girrCreateSchemaLocation"));
    }

    
    public void setGirrCreateSchemaLocation(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("girrCreateSchemaLocation"));
        if (oldValue != val) {
            props.setProperty("girrCreateSchemaLocation", Boolean.toString(val));
            needSave = true;
            firePropertyChange("girrCreateSchemaLocation", oldValue, val);
        }
    }

    public void addGirrCreateSchemaLocationChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("girrCreateSchemaLocation", listener);
    }

    public void removeGirrCreateSchemaLocationChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("girrCreateSchemaLocation", listener);
    }
    
    public String getGirrStyleSheetUrl() {
        return props.getProperty("girrStyleSheetUrl");
    }

    
    public void setGirrStyleSheetUrl(String str) {
        String oldValue = props.getProperty("girrStyleSheetUrl");
        if (!oldValue.equals(str)) {
            props.setProperty("girrStyleSheetUrl", str);
            firePropertyChange("girrStyleSheetUrl", oldValue, str);
            needSave = true;
        }
    }

    public void addGirrStyleSheetUrlChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("girrStyleSheetUrl", listener);
    }

    public void removeGirrStyleSheetUrlChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("girrStyleSheetUrl", listener);
    }
    
    public String getGirrStyleSheetType() {
        return props.getProperty("girrStyleSheetType");
    }

    
    public void setGirrStyleSheetType(String str) {
        String oldValue = props.getProperty("girrStyleSheetType");
        if (!oldValue.equals(str)) {
            props.setProperty("girrStyleSheetType", str);
            firePropertyChange("girrStyleSheetType", oldValue, str);
            needSave = true;
        }
    }

    public void addGirrStyleSheetTypeChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("girrStyleSheetType", listener);
    }

    public void removeGirrStyleSheetTypeChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("girrStyleSheetType", listener);
    }
    /** @return If true (which is the normal case), it is assumed the actual frequency in the imported wave file is half of the actual modulation frequency */
    public boolean getImportWaveDivideCarrier() {
        return Boolean.parseBoolean(props.getProperty("importWaveDivideCarrier"));
    }

    /** @param val If true (which is the normal case), it is assumed the actual frequency in the imported wave file is half of the actual modulation frequency */
    public void setImportWaveDivideCarrier(boolean val) {
        boolean oldValue = Boolean.parseBoolean(props.getProperty("importWaveDivideCarrier"));
        if (oldValue != val) {
            props.setProperty("importWaveDivideCarrier", Boolean.toString(val));
            needSave = true;
            firePropertyChange("importWaveDivideCarrier", oldValue, val);
        }
    }

    public void addImportWaveDivideCarrierChangeListener(IPropertyChangeListener listener) {
        addPropertyChangeListener("importWaveDivideCarrier", listener);
    }

    public void removeImportWaveDivideCarrierChangeListener(IPropertyChangeListener listener) {
        removePropertyChangeListener("importWaveDivideCarrier", listener);
    }
    
    /**
     * Main routine for testing and debugging.
     * @param args filename
     */
    public static void main(String[] args) {
        String filename = args.length > 0 ? args[0] : null;
        try {
            Props p = new Props(filename);
            p.list();
            p.save();
        } catch (IOException e) {
            System.err.println(e.getMessage());
        }
    }
}
