/*
Copyright (C) 2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.irscrutinizer.importer;

import java.io.Serializable;

/**
 * This class does something interesting and useful. Or not...
 *
 * Columns are numbered starting with 1.
 */
public abstract class CsvImporter extends RemoteSetImporter implements IFileImporter,Serializable {
    protected final static int invalid = -1;

    protected String separator;
    protected int nameColumn;
    protected boolean verbose;

    protected int lineNo;

    private final static String[][] separators = new String[][] {
        new String[] { ", (comma)", "," },
        new String[] { "; (semicolon)", ";" },
        new String[] { "TAB", "\t" },
        new String[] { "WHITESPACE", "\\s+" }
    };

    /**
     * @param separator the separator to set
     */
    public void setSeparator(String separator) {
        this.separator = separator;
    }

    public void setSeparatorIndex(int separatorIndex) {
        this.separator = getSeparator(separatorIndex);
    }

    /**
     * @param nameColumn the nameColumn to set
     */
    public void setNameColumn(int nameColumn) {
        this.nameColumn = nameColumn;
    }

    /**
     * @param verbose the verbose to set
     */
    public void setVerbose(boolean verbose) {
        this.verbose = verbose;
    }

    public static String[] separatorsArray() {
        String[] result = new String[separators.length];
        for (int i = 0; i < separators.length; i++)
            result[i] = separators[i][0];
        return result;
    }

    public static String getSeparator(int index){
        return separators[index][1];
    }

    public CsvImporter(String separator, int nameColumn, boolean verbose) {
        super();
        this.separator = separator;
        this.nameColumn = nameColumn;
        this.verbose = verbose;
    }

    protected static String gobbleString(String[] chunks, int column, String aPriori, boolean includeTail) {
        if (column <= 0)
            return aPriori;
        if (column > chunks.length)
            return null;

        StringBuilder str = new StringBuilder(chunks[column-1]);
        if (str.charAt(str.length()-1) == '"')
            str.deleteCharAt(str.length()-1);
        if (str.charAt(0) == '"')
            str.deleteCharAt(0);

        if (includeTail)
            for (int c = column; c < chunks.length; c++)
                str.append(" ").append(chunks[c]);

        return str.toString();
    }

    @Override
    public String[][] getFileExtensions() {
        return new String[][]{ new String[]{ "Text files (*.txt *.text)", "txt", "text" }, { "CVS files (*.csv *.txt *.tsv)", "csv", "txt", "tsv"}};
    }

    @Override
    public boolean canImportDirectories() {
        return false;
    }

    @Override
    public String getFormatName() {
        return "Text (csv)";
    }
}
