/*
Copyright (C) 2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.irscrutinizer.sendinghardware;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import javax.swing.JPanel;
import org.harctoolbox.guicomponents.GuiUtils;
import org.harctoolbox.guicomponents.SerialPortSimpleBean;
import org.harctoolbox.harchardware.IHarcHardware;
import org.harctoolbox.harchardware.ir.IRawIrSender;
import org.harctoolbox.irscrutinizer.Props;

/**
 * This class does something interesting and useful. Or not...
 * @param <T>
 */
public class SendingSerial<T extends IRawIrSender & IHarcHardware> extends SendingHardware<T> implements ISendingHardware<T> {
    private SerialPortSimpleBean serialPortSimpleBean;
    private String portName = null;
    private String initialPort = null;
    private Class<T> clazz;

    public SendingSerial(final Class<T> clazz, JPanel panel, SerialPortSimpleBean serialPortBean, Props props, GuiUtils guiUtils_) {
        super(panel, props, guiUtils_);
        try {
            this.initialPort = (String) Props.class.getMethod("get" + clazz.getSimpleName() + "PortName").invoke(properties);
        } catch (IllegalAccessException ex) {
            guiUtils.error(ex);
        } catch (IllegalArgumentException ex) {
            guiUtils.error(ex);
        } catch (InvocationTargetException ex) {
            guiUtils.error(ex);
        } catch (NoSuchMethodException ex) {
            guiUtils.error(ex);
        } catch (SecurityException ex) {
            guiUtils.error(ex);
        }
        this.serialPortSimpleBean = serialPortBean;
        this.clazz = clazz;
        serialPortBean.addPropertyChangeListener(new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if (propertyName.equals(SerialPortSimpleBean.PROP_VERSION)) {
                    //label.setText((String) evt.getNewValue());
                } else if (propertyName.equals(SerialPortSimpleBean.PROP_PORT)) {
                    portName = (String) evt.getNewValue();
                    if (portName == null)
                        return;

                    try {
                        setupSerial();
                        Props.class.getMethod("set" + clazz.getSimpleName() + "PortName", String.class).invoke(properties, portName);
                        //properties.setIrToyPortName(portName);
                    } catch (IOException ex) {
                        guiUtils.error(ex);
                    } catch (NoSuchMethodException ex) {
                        guiUtils.error(ex);
                    } catch (SecurityException ex) {
                        guiUtils.error(ex);
                    } catch (IllegalAccessException ex) {
                        guiUtils.error(ex);
                    } catch (IllegalArgumentException ex) {
                        guiUtils.error(ex);
                    } catch (InvocationTargetException ex) {
                        guiUtils.error(ex);
                    }
                } else {
                    guiUtils.error("Unknown property " + propertyName);
                }
            }
        });
    }

    private void setupSerial() throws IOException {
        String newPort = serialPortSimpleBean.getPortName();
        if (newPort == null || (rawIrSender != null && newPort.equals(portName)))
            return;

        if (rawIrSender != null)
            rawIrSender.close();
        rawIrSender = null;
        //label.setText("-");
        try {
            rawIrSender = clazz.getConstructor(String.class, int.class, boolean.class).newInstance(newPort, properties.getSendingTimeout(), properties.getVerbose());
            //label.setText(rawIrSender.getVersion());
            portName = newPort;
            Props.class.getMethod("set" + clazz.getSimpleName() + "PortName", String.class).invoke(properties, portName);
            //properties.setIrToyPortName(portName);
            serialPortSimpleBean.setHardware(rawIrSender);
        } catch (NoSuchMethodException ex) {
            guiUtils.error(ex);
        } catch (SecurityException ex) {
            guiUtils.error(ex);
        } catch (InstantiationException ex) {
            guiUtils.error(ex);
        } catch (IllegalAccessException ex) {
            guiUtils.error(ex);
        } catch (IllegalArgumentException ex) {
            guiUtils.error(ex);
        } catch (InvocationTargetException ex) {
            guiUtils.error(ex.getCause().getClass().getName() + " " + ex.getCause().getMessage());
        }
        //portName = serialPortBean.getPortName();
        //serialPortBean.setHardware(rawIrSender);
    }

    @Override
    public void setup() throws IOException {
        serialPortSimpleBean.setup(initialPort);
        initialPort = null;
        setupSerial();
    }

    @Override
    public String getName() {
        return clazz.getSimpleName();
    }
}
